﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Bill.Form', {
    extend: 'System.Control.EntityForm',
    xtype: 'App.Control.Kivii.Finances.Bill.Form',
    entityType: 'Kivii.Finances.Entities.Bill',
    scrollable: true,
    layout: {
        type: 'responsivecolumn',
        states: {
            small: 600,
            large: 0
        }
    },
    title: '账单管理',
    iconCls: 'x-fa fa-file-text-o',
    viewModel: {
        data: {
            entity: {
                Metadata: {},
                Kvid: null,
                Amount: 0,
                AmountPlan: 0,
                AmountPayment: 0,
                OwnerName: KiviiContext.CurrentMember.DepartmentName,
                OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
                OperateTime: new Date(),
                PayeeKvid: KiviiContext.CurrentMember.OrganizationKvid,
                PayeeName: KiviiContext.CurrentMember.OrganizationName,
                OperatorName: KiviiContext.CurrentMember.FullName,
                OperatorKvid: KiviiContext.CurrentMember.Kvid,
                Currency: 'CNY',
                Details: []
            }
        },
        formulas: {
            amountChineseBig: function(get) {
                let _amount = get('entity.Amount');
                return KiviiContext.Const.FinancesFunctionsMoneyToChineseBig(_amount);
            },
            description: function(get) {
                let _kvid = get('entity.Kvid');
                let _amount = get('entity.Amount');
                let _amountPayment = get('entity.AmountPayment');
                if (_kvid === null) return '';
                if (_amount === _amountPayment) return '<div style="color:green">已完成收款登记</div>';
                return '<div style="color:red">未完成收款登记</div>';
            },
            hiddenPaying: function(get) {
                let _kvid = get('entity.Kvid');
                let _amount = get('entity.Amount');
                let _amountPayment = get('entity.AmountPayment');
                if (_kvid === null) return true;
                if (_amount === _amountPayment) return true;
                return false;
            }
        }
    },
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
            margin: '0 3 -10 3',
            xtype: 'displayfield',
            bind: {
                value: '{description}'
            },
            fieldStyle: {
                fontSize: '20px'
            }
        }, '->', {
            text: '创建',
            iconCls: 'x-fa fa-plus',
            iconAlign: 'left',
            handler: '_onSaveEntity',
            bind: {
                disabled: '{!formIsValid}',
                hidden: '{entity.Kvid}'
            }
        }, {
            text: '收款登记',
            iconCls: 'x-fa fa-tags',
            handler: '_onPaying',
            bind: {
                hidden: '{hiddenPaying}'
            }
        }, {
            text: '下载',
            iconCls: 'x-fa fa-download',
            handler: '_onDownloadBill',
            bind: {
                hidden: '{!entity.Kvid}'
            }
        }]
    }],
    defaults: {
        xtype: 'textfield',
        labelWidth: 60,
        labelAlign: 'right',
        responsiveCls: 'big-50 small-100'
    },
    items: [{
        xtype: 'windowSelector',
        fieldLabel: '付款单位',
        responsiveCls: 'big-60 small-100',
        editable: false,
        keyField: 'Kvid',
        valueField: 'FullName',
        window: {
            type: 'App.Control.Kivii.Finances.Settlement.Customer.Selector'
        },
        allowBlank: false,
        bind: {
            key: '{entity.PayerKvid}',
            value: '{entity.PayerName}',
            readOnly: '{entity.Kvid}'
        }
    }, {
        xtype: 'windowSelector',
        fieldLabel: '收款单位',
        responsiveCls: 'big-40 small-100',
        readOnly: true,
        editable: false,
        keyField: 'Kvid',
        valueField: 'FullName',
        window: {
            type: 'App.Control.Kivii.Finances.Settlement.Customer.Selector'
        },
        allowBlank: false,
        bind: {
            key: '{entity.PayeeKvid}',
            value: '{entity.PayeeName}'
        }
    }, {
        xtype: 'numberfield',
        fieldLabel: '账单金额',
        hideTrigger: true,
        minValue: 0,
        responsiveCls: 'big-30 small-100',
        step: 0.1,
        allowBlank: false,
        readOnly: true,
        bind: {
            value: '{entity.Amount}'
        }
    }, {
        xtype: 'numberfield',
        fieldLabel: '已收金额',
        hideTrigger: true,
        minValue: 0,
        responsiveCls: 'big-30 small-100',
        step: 0.1,
        allowBlank: false,
        readOnly: true,
        bind: {
            value: '{entity.AmountPayment}'
        }
    }, {
        xtype: 'categoryField',
        fieldLabel: '货币单位',
        allowBlank: false,
        responsiveCls: 'big-20 small-100',
        internalCode: 'Currency',
        bind: {
            value: '{entity.Currency}',
            readOnly: '{entity.Kvid}'
        }
    }, {
        xtype: 'datefield',
        format: 'Y-m-d',
        altFormats: 'D, d M Y H:i:s T|c|Y-m-d|Y-m-d H:i|Y/n/j',
        fieldLabel: '操作日期',
        editable: false,
        responsiveCls: 'big-20 small-100',
        allowBlank: false,
        bind: {
            value: '{entity.OperateTime}',
            readOnly: '{entity.Kvid}'
        }
    }, {
        responsiveCls: 'big-60 small-100',
        fieldLabel: '金额大写',
        readOnly: true,
        bind: {
            value: '{amountChineseBig}'
        }
    }, {
        fieldLabel: '操&nbsp;&nbsp;作&nbsp;人',
        responsiveCls: 'big-40 small-100',
        allowBlank: false,
        readOnly: true,
        bind: '{entity.OperatorName}'
    }, {
        fieldLabel: '摘&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;要',
        responsiveCls: 'big-100 small-100',
        bind: {
            value: '{entity.Summary}',
            readOnly: '{entity.Kvid}'
        }
    }, {
        xtype: 'textarea',
        fieldLabel: '备&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;注',
        responsiveCls: 'big-100 small-100',
        bind: {
            value: '{entity.Remark}',
            readOnly: '{entity.Kvid}'
        }
    }, {
        xtype: 'fieldset',
        title: '账单明细',
        collapsible: true,
        collapsed: false,
        scrollable: true,
        responsiveCls: 'big-100 small-100',
        items: [{
            xtype: 'grid',
            reference: 'details',
            store: {
                pageSize: 10000,
                groupField: 'Type',
                autoLoad: false
            },
            features: [{
                ftype: 'groupingsummary',
                groupHeaderTpl: [
                    '<div>{name:this.formatName} ({rows.length} 条)</div>',
                    {
                        formatName: function(name) {
                            if (name === "Report") {
                                return "报告费";
                            }
                            if (name === "Service") {
                                return "服务费";
                            }
                            if (name === "Express") {
                                return "快递费";
                            }
                            return name;
                        }
                    }
                ]
            }],
            dockedItems: [{
                xtype: 'toolbar',
                dock: 'top',
                overflowHandler: 'menu',
                items: [{
                    margin: '0 3 -10 3',
                    xtype: 'displayfield',
                    value: '账单明细',
                    fieldStyle: {
                        fontSize: '20px'
                    }
                }, '->', {
                    text: '添加',
                    iconCls: 'x-fa fa-plus',
                    handler: '_onAddDetail',
                    bind: {
                        hidden: '{entity.Kvid}'
                    }
                }]
            }],
            columns: {
                defaults: {
                    menuDisabled: true,
                    sortable: false
                },
                items: [{
                        xtype: 'rownumberer'
                    }, {
                        xtype: 'actioncolumn',
                        align: 'center',
                        text: '操作',
                        items: [{
                            text: '移除',
                            iconCls: 'x-fa fa-trash-o',
                            handler: '_onRemoveDetail'
                        }],
                        width: 80
                    }, {
                        dataIndex: 'BizId',
                        text: '所属业务',
                        width: 150,
                        expend: true,
                        summaryType: 'sum',
                        summaryRenderer: function(value, summaryData, dataIndex) {
                            return '合计 ';
                        }
                    },
                    {
                        dataIndex: 'PayerName',
                        text: '付款单位',
                        width: 150,
                        expend: true
                    },
                    // {
                    //     dataIndex: 'Type',
                    //     text: '类型',
                    //     width: 90,
                    //     align: 'right',
                    //     renderer: function(_value, _meta, _record, _rowIndex) {
                    //         if (_value === "Report") {
                    //             return "检测费";
                    //         }
                    //         if (_value === "Service") {
                    //             return "服务费";
                    //         }
                    //         if (_value === "Express") {
                    //             return "快递费";
                    //         }
                    //         return _value;
                    //     }
                    // },
                    {
                        dataIndex: 'AmountPlan',
                        text: '原价',
                        width: 90,
                        align: 'right',
                        summaryType: 'sum',
                        summaryRenderer: function(value, summaryData, dataIndex) {
                            return Ext.String.format('￥{0}', value.toFixed(2));
                        }
                    }, {
                        dataIndex: 'Amount',
                        text: '折后价',
                        width: 90,
                        align: 'right',
                        summaryType: 'sum',
                        summaryRenderer: function(value, summaryData, dataIndex) {
                            return Ext.String.format('￥{0}', value.toFixed(2));
                        }
                    }, {
                        dataIndex: 'AmountPayment',
                        text: '已付金额',
                        width: 100,
                        align: 'right',
                        summaryType: 'sum',
                        summaryRenderer: function(value, summaryData, dataIndex) {
                            return Ext.String.format('￥{0}', value.toFixed(2));
                        }
                    }, {
                        dataIndex: 'OperatorName',
                        text: '操作人',
                        width: 90
                    }, {
                        dataIndex: 'Summary',
                        text: '摘要',
                        width: 150
                    }, {
                        dataIndex: 'Remark',
                        text: '备注',
                        minWidth: 150,
                        flex: 1
                    }
                ]
            }
        }]
    }],
    setOwner: function(_entity) {
        let _view = this;
        let _vm = _view.getViewModel();
        let _refs = _view.getReferences();
        let _store = _refs.details.getStore();
        _view.readEntity(_entity,false);
        if(Ext.isArray(_entity.Details)){
	    	for (let _i = 0; _i < _entity.Details.length; _i++) {
	            let _record = _entity.Details[_i];
	            _store.add(_record);
	        }
	        _view.getController()._reCalculateAmount();
        }
        if (_entity.Kvid !== null) {
        	_store.removeAll();
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Read.json',//'/Restful/Kivii.Finances.Entities.Bill/Read.json',
                params: {
                    Kvid: _entity.Kvid,
                    IncludeDetail: true
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    _entity=_rtns.Result;
                    _view.readEntity(_rtns.Result, false);
                    for (let _i = 0; _i < _entity.Details.length; _i++) {
                        let _record = _entity.Details[_i];
                        _store.add(_record);
                    }
                },
                failure: function(_response) {}
            });
        }
    },
    controller: {
        _reCalculateAmount: function() {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _refs = _controller.getReferences();
            let _details = _refs.details.getStore().getData().items;
            let _amount = 0;
            let _amountPlan = 0;
            let _amountPayment = 0;
            for (let _i = 0; _i < _details.length; _i++) {
                let _record = _details[_i].data;
                _amount += _record.Amount;
                _amountPlan += _record.AmountPlan;
                _amountPayment += _record.AmountPayment;
            }
            _vm.set('entity.AmountPayment', parseFloat(_amountPayment.toFixed(2)));
            _vm.set('entity.AmountPlan', parseFloat(_amountPlan.toFixed(2)));
            _vm.set('entity.Amount', parseFloat(_amount.toFixed(2)));
        },
        _onAddDetail: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _kvid = _vm.data.entity.Kvid;
            if (_kvid !== null) {
                Ext.toast("已制作账单,无法新增");
                return;
            }
            let _refs = _controller.getReferences();
            let _detailsStore = _refs.details.getStore();
            let _form = Ext.create('App.Control.Kivii.Finances.Bill.Settlement.Selector', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 600,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.on('entitySelected', function(_ctrl, _entities) {
                _win.close();
                let _currents = _detailsStore.getData().items;
                for (let _i = 0; _i < _entities.length; _i++) {
                    if (_currents.length <= 0) _detailsStore.add(_entities[_i]);
                    for (let _j = 0; _j < _currents.length; _j++) {
                        if (_currents[_j].data.Kvid === _entities[_i].Kvid) break;
                        _detailsStore.add(_entities[_i]);
                    }
                }
                _controller._reCalculateAmount();
            });
        },
        _onRemoveDetail: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _kvid = _vm.data.entity.Kvid;
            if (_kvid !== null) {
                Ext.toast("已生成账单,无法移除内容!");
                return;
            }
            _record.drop();
            _controller._reCalculateAmount();
        },
        _onSaveEntity: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _item = _vm.data.entity;
            let _settlementKvids = [];
            let _details = _refs.details.getStore().getData().items;
            if (_details.length <= 0) {
                Ext.toast("无账单明细,无法生成账单!");
                return;
            }
            for (let _i = 0; _i < _details.length; _i++) {
                let _record = _details[_i].data;
                _settlementKvids.push(_record.Kvid);
            }
            let _jsonData = {};
            _jsonData.Item = _item;
            _jsonData.SettlementKvids = _settlementKvids;
            _ctrl.setDisabled(true);
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Create.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('生成账单成功!');
                    _ctrl.setDisabled(false);
                    _view.fireEvent('entityCreated', _view, _rtns.Results);
                },
                failure: function(_response) {
                    Ext.toast('生成账单失败!');
                    _ctrl.setDisabled(false);
                }
            });
        },
        _onPaying: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _refs = _controller.getReferences();
            let _details = _refs.details.getStore().getData().items;
            if (_details.length <= 0) {
                Ext.toast("无账单明细,无法生成账单!");
                return;
            }
            let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Paying.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 900,
                height: 500,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _form.setOwner(_details);
            _win.show();
            _form.on('entityPaying', function(_ctrl, _entity) {
                _win.close();
                Ext.Ajax.request({
                    url: '/Restful/Kivii.Finances.Entities.Bill/Archiving.json',
                    params: {
                        Kvid: _vm.data.entity.Kvid
                    },
                    success: function(_response) {
                        let _rtns = Ext.decode(_response.responseText);
                        if (_rtns.Results.length === 1) {
                            _view.setOwner(_rtns.Results[0]);
                        }
                    },
                    failure: function(_response) {}
                });
            });
        },
        _onDownloadBill:function(_ctrl){
        	let _controller = this;
        	let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _kvid=_vm.data.entity.Kvid;
            let _name=_vm.data.entity.PayerName+"_("+Ext.Date.format(new Date(_vm.data.entity.OperateTime),'Y')+'年'+Ext.Date.format(new Date(_vm.data.entity.OperateTime),'m')+'月账单)';
            
            let _selector = Ext.create('App.Control.Kivii.Finances.Bill.Template.Selector');

            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: '选择模板',
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                autoScroll: true,
                width: 700,
                height: 350,
                modal: true,
                resizable: false,
                items: [_selector]
            });
            _win.show();
            _selector.setFilter({
                Tags: 'Bill'
            });
            _selector.on('entitySelected', function(_ctrl, _entity){
            	_win.close();
            	let _iframe = document.createElement("iframe");
                _iframe.style.display = "none";
                let _form = document.createElement('form');
                
                let _billKvid = document.createElement('input');
                _billKvid.type = 'hidden';
                _billKvid.name = 'Kvid';
                _billKvid.value = _kvid;
                _form.appendChild(_billKvid);
                
                let _includeDetail = document.createElement('input');
                _includeDetail.type = 'hidden';
                _includeDetail.name = 'IncludeDetail';
                _includeDetail.value = true;
                _form.appendChild(_includeDetail);
                
                let _temp = document.createElement('input');
                _temp.type = 'hidden';
                _temp.name = 'View';
                _temp.value = _entity[0].Path;
                _form.appendChild(_temp);

                let _fileName = document.createElement('input');
                _fileName.type = 'hidden';
                _fileName.name = '__fileName';
                _fileName.value = _name;
                _form.appendChild(_fileName);

                _iframe.appendChild(_form);
                document.body.appendChild(_iframe);
                _form.method = 'GET';
                _form.action = '/Restful/' + _view.entityType + '/Read.xlsx';
                _form.submit();
                document.body.removeChild(_iframe);
            });
        }
    }
});