﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Payment.Used.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.Control.Kivii.Finances.Payment.Used.List',
    dataSource: '/Restful/Kivii.Finances.Entities.Payment/UsedQuery.json',
    title: '使用记录',
    iconCls: 'x-fa fa-sign-in',
    filter: {
    	AmountNotEqualTo:0,
        OperateTimeGreaterThanOrEqualTo: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        OperateTimeLessThan: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1)
    },
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 800,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    features: [{
        ftype: 'summary'
    }],
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
                xtype: 'datefield',
                fieldLabel: '时间范围  开始',
                format: 'Y-m-d',
                labelWidth: 90,
                reference: 'startTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                reference: 'endTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                labelWidth: 60,
                queryField: 'SerialNumber,PayerName,PayeeName,PayeeAccountName,PayeeAccountOwnerName,PayeeAccountSerialNumber',
                // queryField: 'Amount',
                emptyText: '模糊搜索流水号、付款单位、充值账户名、充值账号或名称',
            }, {
                text: '充值撤销',
                iconCls: 'x-fa fa-sign-out',
                handler: '_onTransferCancel',
                bind: {
                    disabled: '{!_selection}'
                }
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '流水号',
            dataIndex: 'SerialNumber',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return '合计 ￥';
            },
            width: 220
        }, {
            text: '发生金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: '_renderAmount',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 110
        }, {
            text: '类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 100
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 150
        }, {
            xtype: 'datecolumn',
            text: '发生日期',
            format: 'Y-m-d H:i',
            dataIndex: 'OperateTime',
            width: 150
        }, {
            text: '充值账户',
            dataIndex: 'PayeeAccountName',
            width: 280
        }, {
            text: '账户所属',
            dataIndex: 'PayeeAccountOwnerName',
            width: 150
        },{
            text: '充值账号',
            dataIndex: 'PayeeAccountSerialNumber',
            width: 180
        }, {
            text: '备注',
            dataIndex: 'Remark',
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate'
    },
    setOwner: function(_owner, _ctrl) {
        let _view = this;
        if(_view.filter===null)_view.setFilter({
        	AmountNotEqualTo:0
        });
        _view.filter.ParentKvid=_owner.Kvid;
        _view.reload();
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if(_record.data.hasOwnProperty('OffsetKvid')&&_record.data.OffsetKvid!==null)_meta.style = 'background-color:#FF0000';
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === 'Transfer') {
                return '充值';
            }
            if (_value === 'DynamicPay') {
                return '收款登记';
            }
            return _value;
        },


        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.OperateTimeGreaterThanOrEqualTo;
            delete _view.filter.OperateTimeLessThan;
            _view.filter.OperateTimeGreaterThanOrEqualTo = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.OperateTimeLessThan = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _onTransferCancel: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;
            let _msg = Ext.create('Ext.window.MessageBox', {
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                closeAction: 'destroy'
            });
            _msg.confirm(_ctrl.text, '您确认要撤销选中的' + _selecteds.length + '条记录？', _controller._offsetPayments, _controller);

            return;
        },
        _offsetPayments: function(_optional) {
            if (_optional !== 'yes') return;
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;
            let _kvids = [];
            for (let _i = 0; _i < _selecteds.length; _i++) {
            	if(_selecteds[_i].data.Type==="DynamicPay")continue;
            	if(_selecteds[_i].data.hasOwnProperty('OffsetKvid')&&_selecteds[_i].data.OffsetKvid!==null)continue;
                _kvids.push(_selecteds[_i].data.Kvid);
            }
            if(_kvids.length<=0){
            	Ext.toast("无符合撤销的数据,请重新选择!");
            	return;
            }
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Payment/TransferCancel.json',
                jsonData: {
                    PaymentKvids: _kvids
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('撤销成功！');
                    _view.reload();
                },
                failure: function(_response) {
                    _me.unmask();
                }
            });
        }
    }
});