﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Settlement.Form', {
    extend: 'System.Control.EntityForm',
    xtype: 'App.Control.Kivii.Finances.Settlement.Form',
    entityType: 'Kivii.Finances.Entities.Settlement',
    scrollable: true,
    layout: {
        type: 'responsivecolumn',
        states: {
            small: 600,
            large: 0
        }
    },
    title: '结算管理',
    iconCls: 'x-fa fa-newspaper-o',
    viewModel: {
        data: {
            entity: {
                Metadata: {},
                Kvid: null,
                OffsetKvid: null,
                BizId: '',
                BizType: 'Kivii.Finances.Entities.Settlement',
                Amount: 0,
                AmountPlan: 0,
                AmountPayment: 0,
                OwnerName: KiviiContext.CurrentMember.DepartmentName,
                OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
                OperateTime: new Date(),
                PayeeKvid: KiviiContext.CurrentMember.OrganizationKvid,
                PayeeName: KiviiContext.CurrentMember.OrganizationName,
                OperatorName: KiviiContext.CurrentMember.FullName,
                OperatorKvid: KiviiContext.CurrentMember.Kvid,
                Currency: 'CNY',
                Details: []
            }
        },
        formulas: {
            amountChineseBig: function(get) {
                let _amount = get('entity.Amount');
                return KiviiContext.Const.FinancesFunctionsMoneyToChineseBig(_amount);
            },
            hiddenDelete: function(get) {
                let _kvid = get('entity.Kvid');
                let _offsetKvid = get('entity.OffsetKvid');
                let _amount = get('entity.Amount');
                let _amountPayment = get('entity.AmountPayment');
                if (_kvid === null) return true;
                if (_offsetKvid !== null) return true;
                if (_amountPayment <= 0) return false;
                return true;
            },
            hiddenOffset: function(get) {
                let _kvid = get('entity.Kvid');
                let _offsetKvid = get('entity.OffsetKvid');
                let _amount = get('entity.Amount');
                let _amountPayment = get('entity.AmountPayment');
                if (_kvid === null) return true;
                if (_offsetKvid !== null) return true;
                if (_amount === _amountPayment) return false;
                return true;
            },
            description: function(get) {
                let _offsetKvid = get('entity.OffsetKvid');
                let _bizId = get('entity.BizId');
                let _amount = get('entity.Amount');
                let _amountPayment = get('entity.AmountPayment');
                let _description = '';
                if (_amount === _amountPayment) _description = '<div><p style="color:#B4EEB4">所属业务:' + _bizId + '</p>';
                else _description = '<div><p style="color:#F0E68C">所属业务:' + _bizId + '</p>';
                if (_offsetKvid !== null) _description += ' <p style="color:#FF0000">当前结算已作废!</p></div>';
                else _description += '</div>';
                return _description;
            }
        }
    },
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
            margin: '0 3 -10 3',
            xtype: 'displayfield',
            bind: {
                value: '{description}'
            },
            fieldStyle: {
                fontSize: '20px'
            }

        }, '->', {
            text: '结算',
            iconCls: 'x-fa fa-link',
            iconAlign: 'left',
            handler: '_onSaveEntity',
            bind: {
                disabled: '{!formIsValid}',
                hidden: '{entity.Kvid}'
            }
        }, {
            text: '删除',
            iconCls: 'x-fa fa-trash-o',
            iconAlign: 'left',
            handler: '_onDeleteEntity',
            bind: {
                disabled: '{!formIsValid}',
                hidden: '{hiddenDelete}'
            }
        }, {
            text: '作废',
            iconCls: 'x-fa fa-trash',
            iconAlign: 'left',
            handler: '_onOffsetEntity',
            bind: {
                disabled: '{!formIsValid}',
                hidden: '{hiddenOffset}'
            }
        }]
    }],
    defaults: {
        xtype: 'textfield',
        labelWidth: 60,
        labelAlign: 'right',
        responsiveCls: 'big-50 small-100'
    },
    items: [{
            fieldLabel: '所属部门',
            responsiveCls: 'big-60 small-100',
            allowBlank: false,
            readOnly: true,
            bind: '{entity.OwnerName}'
        }, {
            fieldLabel: '所属业务',
            allowBlank: false,
            responsiveCls: 'big-40 small-100',
            bind: {
                value: '{entity.BizId}',
                readOnly: '{entity.Kvid}'
            }
            // }, {
            //     xtype: 'windowSelector',
            //     fieldLabel: '所属业务',
            //     editable: true,
            //     allowBlank: false,
            //     keyField: 'FOLDERNO',
            //     valueField: 'NAME',
            //     window: {
            //         type: 'App.Control.Kivii.Finances.Settlement.Biz.Report.Selector',
            //         width:1200,
            //         height:600
            //     },
            //     reference: 'BizId',
            //     responsiveCls: 'big-40 small-100',
            //     bind: {
            //         key: '{entity.FOLDERNO}',
            //         value: '{entity.BizId}',
            //         readOnly: '{entity.Kvid}'
            //     },
            //     listeners: {
            //         entitySelected: '_selectBiz'
            //         // entityCleared: '_clearBiz'
            //     }
        }, {
            xtype: 'categoryField',
            fieldLabel: '结算类型',
            allowBlank: false,
            responsiveCls: 'big-30 small-100',
            internalCode: 'Type',
            bind: {
                value: '{entity.Type}',
                readOnly: '{entity.Kvid}'
            }
        }, {
            xtype: 'datefield',
            format: 'Y-m-d',
            altFormats: 'D, d M Y H:i:s T|c|Y-m-d|Y-m-d H:i|Y/n/j',
            fieldLabel: '操作日期',
            editable: false,
            responsiveCls: 'big-30 small-100',
            allowBlank: false,
            bind: {
                value: '{entity.OperateTime}',
                readOnly: '{entity.Kvid}'
            }
        }, {
            fieldLabel: '流&nbsp;&nbsp;水&nbsp;号',
            responsiveCls: 'big-40 small-100',
            bind: {
                value: '{entity.SerialNumber}',
                readOnly: '{entity.Kvid}'
            }
        }, {
            xtype: 'windowSelector',
            fieldLabel: '付款单位',
            responsiveCls: 'big-60 small-100',
            editable: true,
            keyField: 'Kvid',
            valueField: 'FullName',
            window: {
                type: 'App.Control.Kivii.Finances.Settlement.Customer.Selector'
            },
            allowBlank: false,
            bind: {
                key: '{entity.PayerKvid}',
                value: '{entity.PayerName}',
                readOnly: '{entity.Kvid}'
            }
        }, {
            xtype: 'windowSelector',
            fieldLabel: '收款单位',
            responsiveCls: 'big-40 small-100',
            readOnly: true,
            editable: false,
            keyField: 'Kvid',
            valueField: 'FullName',
            window: {
                type: 'App.Control.Kivii.Finances.Settlement.Customer.Selector'
            },
            allowBlank: false,
            bind: {
                key: '{entity.PayeeKvid}',
                value: '{entity.PayeeName}'
            }
        }, {
            xtype: 'numberfield',
            fieldLabel: '结算原价',
            hideTrigger: true,
            minValue: 0,
            responsiveCls: 'big-30 small-100',
            step: 0.1,
            allowBlank: false,
            readOnly: true,
            bind: {
                value: '{entity.AmountPlan}'
            }
        }, {
            xtype: 'numberfield',
            fieldLabel: '折&nbsp;&nbsp;后&nbsp;价',
            hideTrigger: true,
            minValue: 0,
            responsiveCls: 'big-30 small-100',
            step: 0.1,
            allowBlank: false,
            readOnly: true,
            bind: {
                value: '{entity.Amount}'
            }
        }, {
            xtype: 'numberfield',
            fieldLabel: '已收款额',
            hideTrigger: true,
            minValue: 0,
            responsiveCls: 'big-20 small-100',
            step: 0.1,
            allowBlank: false,
            readOnly: true,
            bind: {
                value: '{entity.AmountPayment}'
            }
        }, {
            xtype: 'categoryField',
            fieldLabel: '货币单位',
            allowBlank: false,
            responsiveCls: 'big-20 small-100',
            internalCode: 'Currency',
            bind: {
                value: '{entity.Currency}',
                readOnly: '{entity.Kvid}'
            }
        }, {
            responsiveCls: 'big-60 small-100',
            fieldLabel: '金额大写',
            readOnly: true,
            bind: {
                value: '{amountChineseBig}'
            }
        }, {
            fieldLabel: '操&nbsp;&nbsp;作&nbsp;人',
            responsiveCls: 'big-40 small-100',
            allowBlank: false,
            readOnly: true,
            bind: '{entity.OperatorName}'
        }, {
            fieldLabel: '摘&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;要',
            responsiveCls: 'big-100 small-100',
            bind: {
                value: '{entity.Summary}',
                readOnly: '{entity.Kvid}'
            }
        }, {
            xtype: 'textarea',
            fieldLabel: '备&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;注',
            responsiveCls: 'big-100 small-100',
            bind: {
                value: '{entity.Remark}',
                readOnly: '{entity.Kvid}'
            }
        }, {
            xtype: 'fieldset',
            title: '结算明细',
            collapsible: true,
            collapsed: false,
            scrollable: true,
            responsiveCls: 'big-100 small-100',
            items: [{
                xtype: 'grid',
                reference: 'details',
                store: {
                    pageSize: 10000,
                    data: [],
                    groupField: 'Type',
                    autoLoad: false
                },
                features: [{
                    ftype: 'groupingsummary',
                    groupHeaderTpl: [
                        '<div>{name:this.formatName} ({rows.length} 条)</div>',
                        {
                            formatName: function(name) {
                            	if (name === "Contract") {
                                    return "合同费";
                                }
                                if (name === "ReportItem") {
                                    return "检测费";
                                }
                                if (name === "Service") {
                                    return "服务费";
                                }
                                if (name === "Express") {
                                    return "快递费";
                                }
                                return name;
                            }
                        }
                    ]
                }],
                dockedItems: [{
                    xtype: 'toolbar',
                    dock: 'top',
                    overflowHandler: 'menu',
                    items: [{
                        margin: '0 3 -10 3',
                        xtype: 'displayfield',
                        value: '结算明细',
                        fieldStyle: {
                            fontSize: '20px'
                        }
                    }, '->', {
                        text: '添加',
                        iconCls: 'x-fa fa-plus',
                        handler: '_onAddDetail',
                        bind: {
                            hidden: '{entity.Kvid}'
                        }
                    }]
                }],
                columns: {
                    defaults: {
                        menuDisabled: true,
                        sortable: false
                    },
                    items: [{
                            xtype: 'rownumberer'
                        }, {
                            xtype: 'actioncolumn',
                            align: 'center',
                            text: '操作',
                            items: [{
                                text: '编辑',
                                iconCls: 'x-fa fa-pencil-square-o',
                                handler: '_onEditDetail'
                            }, '->', {
                                text: '移除',
                                iconCls: 'x-fa fa-trash-o',
                                handler: '_onRemoveDetail'
                            }],
                            width: 100
                        }, {
                            dataIndex: 'GoodsId',
                            text: '条目编号',
                            width: 120
                        }, {
                            dataIndex: 'GoodsFullName',
                            text: '条目名称',
                            width: 150,
                            expend: true,
                            summaryType: 'sum',
                            summaryRenderer: function(value, summaryData, dataIndex) {
                                return '合计 ';
                            }
                        },
                        {
                            dataIndex: 'Type',
                            text: '类型',
                            width: 90,
                            align: 'right',
                            renderer: function(_value, _meta, _record, _rowIndex) {
                            	if (_value === "Contract") {
                                    return "合同费";
                                }
                                if (_value === "ReportItem") {
                                    return "检测费";
                                }
                                if (_value === "Service") {
                                    return "服务费";
                                }
                                if (_value === "Express") {
                                    return "快递费";
                                }
                                return _value;
                            }
                        },
                        {
                            dataIndex: 'AmountPlan',
                            text: '原价',
                            width: 90,
                            align: 'right',
                            summaryType: 'sum',
                            summaryRenderer: function(value, summaryData, dataIndex) {
                                return Ext.String.format('￥{0}', value.toFixed(2));
                            }
                        }, {
                            dataIndex: 'Amount',
                            text: '折后价',
                            width: 90,
                            align: 'right',
                            summaryType: 'sum',
                            summaryRenderer: function(value, summaryData, dataIndex) {
                                return Ext.String.format('￥{0}', value.toFixed(2));
                            }
                        }, {
                            dataIndex: 'Quantity',
                            text: '结算数量',
                            renderer: function(_value, _meta, _record, _rowIndex) {
                                let _rtns = _value;
                                if (_record.data.hasOwnProperty('QuantityUnit')) _rtns = _value + ' ' + _record.data.QuantityUnit;
                                return _rtns;
                            },
                            width: 100
                        }, {
                            dataIndex: 'GoodsSpecifications',
                            text: '规格',
                            width: 150
                        }, {
                            dataIndex: 'GoodsModel',
                            text: '型号',
                            width: 120
                        }, {
                            dataIndex: 'Remark',
                            text: '备注',
                            minWidth: 150,
                            flex: 1
                        }
                    ]
                }
            }]
        },
        {
            xtype: 'fieldset',
            title: '扩展信息',
            collapsible: true,
            collapsed: true,
            scrollable: true,
            maxHeight: 300,
            responsiveCls: 'big-100 small-100',
            items: [{
                xtype: 'propertygrid',
                reference: 'metadata',
                bind: {
                    source: '{entity.Metadata}'
                },
                tbar: [{
                    margin: '0 3 -10 3',
                    xtype: 'displayfield',
                    value: '扩展信息',
                    fieldStyle: {
                        fontSize: '20px'
                    }
                }, '->', {
                    iconCls: 'x-fa fa-plus',
                    handler: '_onAddMetadata',
                    bind: {
                        hidden: '{entity.Kvid}'
                    }
                }, {
                    iconCls: 'x-fa fa-remove',
                    handler: '_onRemoveMetadata',
                    bind: {
                        hidden: '{entity.Kvid}'
                    }
                }]
            }]
        }
    ],
    setOwner: function(_entity) {
        let _view = this;
        let _vm = _view.getViewModel();
        let _refs = _view.getReferences();
        let _detailsStore = _refs.details.getStore();
        let _controller = _view.getController();
        if (_entity.Kvid !== null) {
            let _jsonData = {};
            _jsonData.EntityType = _entity.BizType;
            _jsonData.Kvid = _entity.Kvid;
            _jsonData.IncludeDetail = true;
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Read.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    _vm.set('entity', _rtns.Result);
                    for (let _i = 0; _i < _rtns.Result.Details.length; _i++) {
                        let _detail = _rtns.Result.Details[_i];
                        _detailsStore.add(_detail);
                    }
                },
                failure: function(_response) {}
            });
        } else {
            _vm.set('entity', _entity);
            for (let _i = 0; _i < _entity.Details.length; _i++) {
                let _detail = _entity.Details[_i];
                _detailsStore.add(_detail);
            }
        }
        _controller._reCalculateAmount();
    },
    controller: {
        _reCalculateAmount: function() {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _refs = _controller.getReferences();
            let _details = _refs.details.getStore().getData().items;
            let _amount = 0;
            let _amountPlan = 0;
            for (let _i = 0; _i < _details.length; _i++) {
                let _record = _details[_i].data;
                _amount += parseFloat(_record.Amount);
                _amountPlan += parseFloat(_record.AmountPlan);
            }
            _vm.set('entity.AmountPlan', parseFloat(_amountPlan.toFixed(2)));
            _vm.set('entity.Amount', parseFloat(_amount.toFixed(2)));
        },

        // _selectBiz: function(_ctrl, _entities) {
        //     let _controller = this;
        //     let _vm = _controller.getViewModel();
        //     let _entity = _entities[0];
        //     _vm.set('entity.Type','Report');

        //     _vm.set('entity.BizType', "Njzj.Connects.Entities.NjzjReport");
        //     _vm.set('entity.BizId', _entity.NAME);
        //     _vm.set('entity.PayerName',_entity.NAMEOFPRODUCTIONORG);
        //     _vm.set('entity.Summary',_entity.SAMPLENAME);
        //     let _metadata = {};
        //     for (let key in _entity) {
        //     	if(key==='Kvid')continue;
        //         _metadata[key] = _entity[key];
        //     }
        //     _vm.set('entity.Metadata', _metadata);


        //     let _refs = _controller.getReferences();
        //     let _detailsStore = _refs.details.getStore();
        //     _detailsStore.removeAll();
        //     Ext.Ajax.request({
        //         url: '/Restful/Njzj.Connects.Entities.NjzjReportItem/Query.json',
        //         jsonData: {
        //             ReportId: _entity.NAME,
        //             IsSerial:true
        //         },
        //         success: function(_response) {
        //             let _rtns = Ext.decode(_response.responseText);
        //             if (_rtns.Results.length <=0)return;
        //             for (let _i = 0; _i < _rtns.Results.length; _i++) {
        //             	let _detail=_rtns.Results[_i];
        //             	_detailsStore.add(_detail);
        //       //     	let _newEntity = {
        //       //     Metadata: {},
        //       //     Type: 'ReportItem',
        //       //     Kvid: null,
        //       //     Amount: 0,
        //       //     AmountPlan: 0,
        //       //     Quantity: 1,
        //       //     QuantityUnit: '次',
        //       //     Currency: 'CNY'
        //       // };
        //       // _newEntity.BizId=_detail.FOLDERNO+'['+_detail.ORDNO+']('+_detail.ANALYTES+')';
        //       // _newEntity.BizType='Njzj.Connects.Entities.NjzjReportItem';
        //       // _newEntity.GoodsFullName=_detail.TESTNO;
        //       // if(_detail.ANALYTES!==_newEntity.GoodsFullName)_newEntity.GoodsFullName+='('+_detail.ANALYTES+')';
        //       // _newEntity.GoodsId=_detail.SERVGRP;
        //       // _newEntity.GoodsSpecifications=_detail.METHODNAME+'('+_detail.METHOD+')';
        //       // _newEntity.GoodsModel=_detail.SAMPLENAME;
        //       // for (let key in _detail) {
        //       //     _newEntity.Metadata[key] = _detail[key];
        //       // }
        //       // _detailsStore.add(_newEntity);
        //             }
        //         	_controller._reCalculateAmount();
        //         },
        //         failure: function(_response) {
        //         }
        //     });
        // },
        // _clearBiz: function(_ctrl) {
        //     let _controller = this;
        //     let _vm = _controller.getViewModel();
        //     let _refs = _controller.getReferences();
        //     let _detailsStore = _refs.details.getStore();
        //     _detailsStore.removeAll();
        //     _controller._reCalculateAmount();
        //     _vm.set('entity.BizId', '');
        //     _vm.set('entity.BizType', '');
        //     _vm.set('entity.Summary','');
        //     _vm.set('entity.Metadata', {});
        // },
        _onAddMetadata: function(_ctrl) {
            let _controller = this;
            Ext.MessageBox.prompt('增加参数名称', '请输入扩展信息参数名称', _controller._onAddMetadataResult, _controller);
        },
        _onAddMetadataResult: function(_btn, _text) {
            if (_btn !== 'ok') return;
            let _controller = this,
                _vm = _controller.getViewModel();
            let _metadata = _vm.data.entity.Metadata;
            if (_metadata[_text]) return;
            _metadata[_text] = '';
            _vm.set('entity.Metadata', _metadata);
            let _refs = _controller.getReferences();
            _refs.metadata.setSource(_metadata);

        },
        _onRemoveMetadata: function(_ctrl) {
            let _controller = this,
                _refs = _controller.getReferences();
            if (!_refs.metadata.selection) return;
            let _vm = _controller.getViewModel();
            let _metadata = _vm.data.entity.Metadata;
            delete _metadata[_refs.metadata.selection.id];
            _vm.set('entity.Metadata', _metadata);
            _refs.metadata.setSource(_metadata);
        },
        _onAddDetail: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _kvid = _vm.data.entity.Kvid;
            if (_kvid !== null) {
                Ext.toast("已结算项目,无法新增");
                return;
            }
            let _refs = _controller.getReferences();
            let _detailsStore = _refs.details.getStore();
            let _newEntity = {
                Metadata: {},
                Type: 'ReportItem',
                Kvid: null,
                Amount: 0,
                AmountPlan: 0,
                Quantity: 1,
                QuantityUnit: '次',
                Currency: 'CNY'
            };
            let _owner = _vm.data.entity;
            let _form = Ext.create('App.Control.Kivii.Finances.SettlementDetail.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 800,
                height: 350,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_newEntity, _owner, _ctrl);
            _form.on('entityAdd', function(_ctrl, _entity) {
                _win.close();
                _detailsStore.add(_entity);
                _controller._reCalculateAmount();
            });
        },
        _onEditDetail: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _kvid = _vm.data.entity.Kvid;
            if (_kvid !== null) {
                Ext.toast("已结算项目,无法编辑");
                return;
            }
            let _refs = _controller.getReferences();
            let _detailsStore = _refs.details.getStore();
            let _owner = _vm.data.entity;
            let _form = Ext.create('App.Control.Kivii.Finances.SettlementDetail.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _item.text,
                iconCls: _item.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 800,
                height: 350,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_record.data, _owner, _item);
            _form.on('entityAdd', function(_ctrl, _entity) {
                _win.close();
                _record.set(_entity, null);
                _detailsStore.commitChanges();
                _controller._reCalculateAmount();
            });
        },
        _onRemoveDetail: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _kvid = _vm.data.entity.Kvid;
            if (_kvid !== null) {
                Ext.toast("已结算项目,无法移除");
                return;
            }
            _record.drop();
            _controller._reCalculateAmount();
        },
        _onSaveEntity: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _item = _vm.data.entity;
            _item.Details = [];
            let _kvid = _vm.data.entity.Kvid;
            if (_kvid !== null) {
                Ext.toast("已结算项目,无法新增!");
                return;
            }
            let _details = _refs.details.getStore().getData().items;
            if (_details.length <= 0) {
                Ext.toast("无结算明细,无法新增!");
                return;
            }
            for (let _i = 0; _i < _details.length; _i++) {
                let _record = _details[_i].data;
                _item.Details.push(_record);
            }
            let _jsonData = {};
            _jsonData.EntityType = _item.BizType;
            _jsonData.Item = _item;
            _ctrl.setDisabled(true);
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Create.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('结算成功!');
                    _ctrl.setDisabled(false);
                    _view.fireEvent('entityCreated', _view, _rtns.Results);
                },
                failure: function(_response) {
                    Ext.toast('结算失败!');
                    _ctrl.setDisabled(false);
                }
            });
        },
        _onDeleteEntity: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _item = _vm.data.entity;
            if (_item.AmountPayment !== 0) {
                Ext.toast('已收款结算,无法删除!');
                return;
            }
            let _msg = Ext.create('Ext.window.MessageBox', {
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                closeAction: 'destroy'
            });
            _msg.confirm("删除确认", "您确认要删除当前的结算信息？", _controller.beforeDeleteEntity, _controller);
        },
        beforeDeleteEntity: function(_optional) {
            if (_optional != "yes") return;
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _item = _vm.data.entity;
            if (_item.AmountPayment !== 0) {
                Ext.toast('已收款结算,无法删除,请作废!');
                return;
            }
            let _jsonData = {};
            _jsonData.EntityType = _item.BizType;
            _jsonData.Kvids = [];
            _jsonData.Kvids.push(_item.Kvid);
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Delete.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('删除成功!');
                    _view.fireEvent('entityDeleted', _view, _rtns.Results);
                },
                failure: function(_response) {
                    Ext.toast('删除失败!');
                }
            });
        },
        _onOffsetEntity: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _item = _vm.data.entity;
            if (_item.AmountPayment !== _item.Amount) {
                Ext.toast('未收款结算,无法作废,请删除!');
                return;
            }
            let _msg = Ext.create('Ext.window.MessageBox', {
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                closeAction: 'destroy'
            });
            _msg.confirm("作废确认", "您确认要作废当前的结算信息？", _controller.beforeOffsetEntity, _controller);
        },
        beforeOffsetEntity: function(_optional) {
            if (_optional != "yes") return;
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _item = _vm.data.entity;
            if (_item.AmountPayment !== _item.Amount) {
                Ext.toast('未收款结算,无法作废,请删除!');
                return;
            }
            let _jsonData = {};
            _jsonData.EntityType = _item.BizType;
            _jsonData.Kvids = [];
            _jsonData.Kvids.push(_item.Kvid);
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Offset.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('删除成功!');
                    _view.fireEvent('entityDeleted', _view, _rtns.Results);
                },
                failure: function(_response) {
                    Ext.toast('删除失败!');
                }
            });
        }
    }
});