﻿Ext.define('App.View.Kivii.Finances.InvoiceApply.Payment.List', {
    extend: 'System.Control.EntitySelector',
    entityType: 'Kivii.Finances.Entities.Payment',
    xtype: 'App.View.Kivii.Finances.InvoiceApply.Payment.List',
    dataSource: '/Restful/Kivii.Finances.Entities.Payment/InvoiceApplyQuery.json',
    title: '到账申请',
    iconCls: 'x-fa fa-money',
    filter: {
    	OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid
    },
    viewModel: {
        data: {
            _selection: null
        }
    },
    store: {
        autoLoad: true,
        pageSize: 100,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    bind: {
        selection: '{_selection}'
    },
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
            margin: '0 3 -10 3',
            xtype: 'displayfield',
            value: '<div style="color:red">若未找到开票收款,请先到账管理中认领</div>',
            fieldStyle: {
                fontSize: '20px'
            }
        }, {
            xtype: 'searchField',
            flex: 1,
            labelWidth: 60,
            queryField: 'SerialNumber,PayerName,PayerAccountName,PayerAccountSerialNumber',
            emptyText: '模糊搜索流水号、付款单位、收付款账号或名称'
        }, {
            text: '到账申请',
            iconCls: 'x-fa fa-money',
            type: 'Payment',
            handler: '_onCreateApply',
            bind: {
                disabled: '{!_selection}'
            }
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '流水号',
            dataIndex: 'SerialNumber',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            width: 220
        }, {
            text: '发生金额',
            align: 'right',
            dataIndex: 'Amount',
            width: 110
        }, {
            text: '开票金额',
            align: 'right',
            dataIndex: 'AmountInvoice',
            width: 100
        }, {
            text: '付款类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 100
        }, {
            xtype: 'datecolumn',
            text: '发生日期',
            format: 'Y-m-d H:i',
            dataIndex: 'OperateTime',
            width: 150
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 150
        }, {
            text: '备注',
            dataIndex: 'Remark',
            minWidth: 150,
            flex: 1
        }]
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === 'Cash') {
                return '现金';
            }
            if (_value === 'Pos') {
                return '刷卡';
            }
            if (_value === 'AliPay') {
                return '支付宝';
            }
            if (_value === 'WeChat') {
                return '微信';
            }
            if (_value === 'Bank') {
                return '银行转账';
            }
            if (_value === 'Split') {
                return '已认领到账';
            }
            return _value;
        },
        _onCreateApply: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;
            let _apply = {};
            _apply.Metadata = {};
            _apply.PaymentKvids = [];
            _apply.OperateType = _ctrl.type;
            _apply.Amount=0;
            _apply.PayerName=null;
            _apply.PayeeName = KiviiContext.CurrentMember.OrganizationName;
            _apply.PayeeTaxNumber = null;
            _apply.OperateTime = new Date();
            _apply.Currency = 'CNY';
            for (let _i = 0; _i < _selecteds.length; _i++) {
                let _select = _selecteds[_i].data;
                _apply.PayerName = _select.PayerName;
                _apply.PaymentKvids.push(_select.Kvid);
                _apply.Amount += parseFloat(parseFloat(_select.Amount - _select.AmountInvoice).toFixed(2));
            }
            let _form = Ext.create('App.Control.Kivii.Finances.InvoiceApply.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 900,
                height: 450,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_apply, _ctrl);
            _form.on('entityCreated', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
    }
});