﻿Ext.define('App.View.Kivii.Finances.Settlement.Biz.Report.List', {
    extend: 'System.Control.EntityList',
    xtype: 'App.View.Kivii.Finances.Settlement.Biz.Report.List',
    dataSource: '/Restful/Njzj.Connects.Entities.NjzjReport/Query.json',
    requires: ['System.Control.ComboSelector'],
    title: '未结算报告',
    iconCls: 'x-fa fa-file-pdf-o',
    filter: {
        BeginTime: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        EndTime: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
    },
    viewModel: {
        data: {
            _selection: null
        },
        formulas: {
            settlementButtonText: function(get) {
                let _selections = get('selections');
                if (_selections === null) return '结算';
                if (_selections.length > 1) return '批量结算';
                return '结算';
            }
        }
    },
    bind: {
        selection: '{_selection}'
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    store: {
        autoLoad: true,
        pageSize: 400
    },
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
                xtype: 'comboSelector',
                dataSource: '/Restful/Njzj.Connects.Entities.NjzjDepartment/Query.json',
                keyField: 'DEPT',
                valueField: 'DEPTNAME',
                width: 350,
                editable: false,
                listeners: {
                    select: '_selectDepart'
                }
            }, {
                xtype: 'datefield',
                fieldLabel: '时间范围  开始',
                format: 'Y-m-d',
                labelWidth: 90,
                reference: 'startTime',
                value: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanBeginOperateTime'
                    }
                },
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                reference: 'endTime',
                value: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
                editable: false,
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanEndOperateTime'
                    }
                },
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'combo',
                queryMode: 'local',
                displayField: 'displayInfo',
                valueField: 'search',
                value: 'PREORDNO',
                editable: false,
                filterPickList: true,
                reference: 'searchType',
                store: {
                    fields: ['displayInfo', 'search'],
                    data: [{
                        displayInfo: '样品编号',
                        search: 'PREORDNO'
                    },{
                        displayInfo: '委托单号',
                        search: 'FOLDERNO'
                    },{
                        displayInfo: '生产单位',
                        search: 'STRPRODUCTIONORG'
                    },{
                        displayInfo: '协议编号',
                        search: 'TESTTASKNOFORCCC'
                    },{
                        displayInfo: '客户类别',
                        search: 'STRCLIENTTYPE'
                    }]
                },
                listeners: {
                    select: '_onSelectSearchType'
                }
            },{
                xtype: 'searchField',
                flex: 1,
                labelWidth: 60,
                reference:'search',
                queryField: 'PREORDNO',
                emptyText: '模糊搜索样品编号'
            }, {
                text: '结算',
                iconCls: 'x-fa fa-link',
                handler: '_onCreateSettlement',
                bind: {
                    text: '{settlementButtonText}',
                    disabled: '{!_selection}'
                }
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            dataIndex: 'RN',
            text: '序号',
            width: 50
        }, {
            dataIndex: 'FOLDERNO',
            text: '委托单号',
            width: 100
        }, {
            dataIndex: 'NAME',
            text: '样品编号',
            width: 170
        }, {
            dataIndex: 'COMPNAME',
            text: '委托单位',
            width: 150
        }, {
            dataIndex: 'NAMEOFPRODUCTIONORG',
            text: '生产单位',
            width: 120
        }, {
            dataIndex: 'SAMPLENAME',
            text: '样品名称',
            width: 120
        }, {
            dataIndex: 'TOTALPRICE',
            text: '应收费用',
            width: 100
        }, {
            dataIndex: 'LOGDATE',
            text: '收样日期',
            width: 100
        // }, {
        //     dataIndex: 'COMPANYNAME',
        //     text: 'Companyname',
        //     width: 150
        }, {
            dataIndex: 'TESTTASKNOFORCCC',
            text: '协议编号',
            flex: 1
        }]
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        //初始化成结算对象方法
        _initNewEntity: function(_entity, _includeDetail = false) {
            let _newEntity = {
                Metadata: {},
                Kvid: null,
                OffsetKvid: null,
                BizId: '',
                Amount: 0,
                AmountPlan: 0,
                AmountPayment: 0,
                Type: 'Report',
                OwnerName: KiviiContext.CurrentMember.DepartmentName,
                OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid,
                OperateTime: new Date(),
                PayeeKvid: KiviiContext.CurrentMember.OrganizationKvid,
                PayeeName: KiviiContext.CurrentMember.OrganizationName,
                OperatorName: KiviiContext.CurrentMember.FullName,
                OperatorKvid: KiviiContext.CurrentMember.Kvid,
                Currency: 'CNY',
                Details: []
            };
            _newEntity.Kvid = null;
            _newEntity.Amount = parseFloat(parseFloat(_entity.TOTALPRICE).toFixed(2));
            _newEntity.AmountPlan = parseFloat(parseFloat(_entity.TOTALPRICE).toFixed(2));
            _newEntity.BizType = 'Kivii.Finances.Entities.Settlement';
            _newEntity.BizId = _entity.NAME;
            _newEntity.PayerName = _entity.COMPNAME;
            _newEntity.Summary = _entity.SAMPLENAME;
            for (let key in _entity) {
                if (key === 'Kvid') continue;
                _newEntity.Metadata[key] = _entity[key];
            }
            if (_includeDetail) {
                let _detail = {};
                _detail.Type = "ReportItem";
                _detail.Quantity = 1;
                _detail.QuantityUnit = "次";
                _detail.Currency = 'CNY';
                _detail.Amount = parseFloat(parseFloat(_entity.TOTALPRICE).toFixed(2));
                _detail.AmountPlan = parseFloat(parseFloat(_entity.TOTALPRICE).toFixed(2));
                _detail.BizId = _entity.NAME;
                _detail.BizType = 'Njzj.Connects.Entities.NjzjReport';
                _detail.GoodsFullName = _entity.SAMPLENAME;
                _detail.GoodsSpecifications = '批量结算明细';
                _newEntity.Details.push(_detail);
            }
            return _newEntity;
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.BeginTime;
            delete _view.filter.EndTime;
            _view.filter.BeginTime = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.EndTime = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _cleanBeginOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            _view.filter.BeginTime = null;
            _view.reload();
        },
        _cleanEndOperateTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            _view.filter.EndTime = null;
            _view.reload();
        },
        _selectDepart: function(_ctrl, _record, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            _view.filter.Dept = _record.data.DEPT;
            _view.reload();
        },
        _onSelectSearchType: function(_ctrl, _record) {
            let _controller = this;
            let _refs=_controller.getReferences();
            let _search=_refs.search;
            _search.queryField=_record.data.search;
            _search.setEmptyText('模糊搜索'+_record.data.displayInfo);
        },
        _onCreateSettlement: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            if (!_view.selection) return;
            if (_vm.data.selections.length === 1) {
                let _selected = _view.selection.data;
                let _newEntity = _controller._initNewEntity(_selected);
                let _form = Ext.create('App.Control.Kivii.Finances.Settlement.Form', {
                    header: false
                });
                let _win = Ext.create('Ext.window.Window', {
                    layout: 'fit',
                    title: _ctrl.text,
                    iconCls: _ctrl.iconCls,
                    iconAlign: 'left',
                    // header:false,
                    autoScroll: true,
                    width: 1300,
                    height: 720,
                    modal: true,
                    maximizable: true,
                    resizable: true,
                    items: [_form]
                });
                _form.on('entityCreated', function(_ctrl, _entity) {
                    _win.close();
                    _view.reload();
                });
                Ext.Ajax.request({
                    url: '/Restful/Njzj.Connects.Entities.NjzjReportItem/Query.json',
                    async: false,
                    jsonData: {
                        ReportId: _newEntity.BizId,
                        IsSerial: true
                    },
                    success: function(_response) {
                        let _rtns = Ext.decode(_response.responseText);
                        if (_rtns.Results.length > 0) {
                            for (let _i = 0; _i < _rtns.Results.length; _i++) {
                                let _detail = _rtns.Results[_i];
                                _newEntity.Details.push(_detail);
                            }
                        }
                        _win.show();
                        _form.setOwner(_newEntity);
                    },
                    failure: function(_response) {}
                });
            } else {
                let _selects = [];
                for (let _i = 0; _i < _vm.data.selections.length; _i++) {
                    _selects.push(_vm.data.selections[_i].data);
                }
                let _newEntities = [];
                for (let _i = 0; _i < _selects.length; _i++) {
                    let _newEntity = _controller._initNewEntity(_selects[_i], true);
                    _newEntities.push(_newEntity);
                }
                let _form = Ext.create('App.Control.Kivii.Finances.Settlement.MultiForm', {
                    header: false
                });
                let _win = Ext.create('Ext.window.Window', {
                    layout: 'fit',
                    title: _ctrl.text,
                    iconCls: _ctrl.iconCls,
                    iconAlign: 'left',
                    // header:false,
                    autoScroll: true,
                    width: 1300,
                    height: 720,
                    modal: true,
                    maximizable: true,
                    resizable: true,
                    items: [_form]
                });
                _win.show();
                _form.setOwner(_newEntities);
                _form.on('entityCreated', function(_ctrl, _entity) {
                    _win.close();
                    _view.reload();
                });
            }
        }
    }
});